--
-- WoodHarvesterTreeInfo for FS17
-- Show info about tree when cutting with woodharvester. Also shows statistics about cutted trees.
-- @author:    	kenny456 (kenny456@seznam.cz)
-- @history:	v1.1 - 2018-07-03 - 
-- 
WoodHarvesterTreeInfo = {};
local WoodHarvesterTreeInfo_directory = g_currentModDirectory;

function WoodHarvesterTreeInfo.prerequisitesPresent(specializations)
    return true;
end;

function WoodHarvesterTreeInfo:load(savegame)
	self.wti = {};
	self.wti.hudInfoPosX = 0.808
	self.wti.hudInfoPosY = 0.203
	self.wti.hudInfoSizeX = 0.169
	self.wti.hudInfoSizeY = 0.057
	self.wti.treeInfo = "";
	self.wti.treeInfo2 = "";
	self.wti.treeInfo3 = "";
	self.wti.treeInfo4 = "";
	self.wti.lasDiameterLoc = 0;
	self.wti.treeFound = false;
	self.wti.statsSession = {};
    self.wti.statsSession['trees'] = {0,0,0,0,0};
    self.wti.statsSession['cuts'] = {0,0,0,0,0};
    self.wti.statsTotal= {};
    self.wti.statsTotal['trees'] = {0,0,0,0,0};
    self.wti.statsTotal['cuts'] = {0,0,0,0,0};
	self.wti.showStats = true
	self.sizeXTemp = 0
	self.massTemp = 0
	self.volTemp = 0
	self.rigidBodyTemp = nil
	self.isCutting = false
	self.wti.writeStats = false
	self.wti.isChipping = false
	self.wti.hud = Overlay:new("WoodHarvesterTreeInfo", Utils.getFilename("WoodHarvesterTreeInfo_hud.dds", WoodHarvesterTreeInfo_directory), self.wti.hudInfoPosX, self.wti.hudInfoPosY, self.wti.hudInfoSizeX, self.wti.hudInfoSizeY);
	if g_currentMission.sawdustBase ~= nil and g_currentMission.sawdustBase ~= 0 then
		self.wti.woodChipsOn = true
	else
		self.wti.woodChipsOn = false
	end
end;

function WoodHarvesterTreeInfo:postLoad(savegame) 
	if savegame ~= nil then
		local i = 0
		while true do
			local statsTotalKey = savegame.key .. string.format(".statsTotal(%d)", i)
			if not hasXMLProperty(savegame.xmlFile, statsTotalKey) then
				break
			end
			local name = getXMLString(savegame.xmlFile, statsTotalKey .. "#name")
			local count = getXMLFloat(savegame.xmlFile, statsTotalKey .. "#count")
			local length = getXMLFloat(savegame.xmlFile, statsTotalKey .. "#length")
			local weight = getXMLFloat(savegame.xmlFile, statsTotalKey .. "#weight")
			local vol = getXMLFloat(savegame.xmlFile, statsTotalKey .. "#vol")
			local woodChips = getXMLFloat(savegame.xmlFile, statsTotalKey .. "#woodChips")
			if name ~= nil and count ~= nil and length ~= nil and weight ~= nil and vol ~= nil and woodChips ~= nil and self.wti.statsTotal[name] ~= nil then
				self.wti.statsTotal[name][1] = count
				self.wti.statsTotal[name][2] = length
				self.wti.statsTotal[name][3] = weight
				self.wti.statsTotal[name][4] = vol
				self.wti.statsTotal[name][5] = woodChips
			end
			i = i + 1
		end    
	end   
end;

function WoodHarvesterTreeInfo:getSaveAttributesAndNodes(nodeIdent)
	local attributes, nodes = "","";
    for k,v in pairs (self.wti.statsTotal) do
        if 0 < nodes.len(nodes) then
            nodes = nodes .. "\n"
        end
        nodes = nodes..nodeIdent..'<statsTotal name="'..k..'" count="'..v[1]..'" length="'..v[2]..'" weight="'..v[3]..'" vol="'..v[4]..'" woodChips="'..v[5]..'"/>';
    end
    return attributes,nodes;
end;

function WoodHarvesterTreeInfo:delete()
end;

function WoodHarvesterTreeInfo:readStream(streamId, connection)
end;

function WoodHarvesterTreeInfo:writeStream(streamId, connection)
end;

function WoodHarvesterTreeInfo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WoodHarvesterTreeInfo:keyEvent(unicode, sym, modifier, isDown)
end;

function WoodHarvesterTreeInfo:update(dt)
	if self:getIsActiveForInput(true) and self:getIsTurnedOn() then
		if self.cutNode ~= nil then
			local x,y,z = getWorldTranslation(self.cutNode);
			local nx,ny,nz = localDirectionToWorld(self.cutNode, 1,0,0);
			local yx,yy,yz = localDirectionToWorld(self.cutNode, 0,1,0);
			local shape, minY, maxY, minZ, maxZ = findSplitShape(x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ);
			if shape ~= 0 or self.hasAttachedSplitShape == true then
				if shape == 0 then
					shape = self.attachedSplitShape
				end
				if self.hasAttachedSplitShape == true then
					shape = self.attachedSplitShape
				end
				self.wti.treeFound = true;
				splitType = SplitUtil.splitTypes[getSplitType(shape)];
				if shape == self.attachedSplitShape then
					self.wti.lasDiameterLoc = self.lastDiameter;
				else
					self.wti.lasDiameterLoc = math.max(maxY-minY, maxZ-minZ);
				end;
				local sizeX, sizeY, sizeZ, numConvexesn, numAttachments = getSplitShapeStats(shape);
				local mass = getMass(shape)
				local vol = getVolume(shape)
				self.wti.treeInfo = self.wti.treeInfo .. "Tree type : " .. splitType.name .. "\n";
				self.wti.treeInfo = self.wti.treeInfo .. "Length :\n";
				self.wti.treeInfo = self.wti.treeInfo .. "Weight :\n";
				if self.wti.showStats == true and self.wti.treeFound == true then
					self.wti.treeInfo = self.wti.treeInfo .. "----------------------------------------------------------\n";
				end
				self.wti.treeInfo2 = self.wti.treeInfo2 .. "\n" .. string.format("%.1f", sizeX) .. " m\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. string.format("%.1f", getMass(shape)) .. " t\n\n";
				
				self.wti.treeInfo3 = self.wti.treeInfo3 .. "\nSize :\nVol.:\n\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. "\n" .. string.format("%.0f", self.wti.lasDiameterLoc * 1000) .. " mm\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. string.format("%.1f", getVolume(shape)) .. " m3\n\n";
				
				if self.wti.writeStats == true then
					self.wti.writeStats = false
					if self.rigidBodyTemp == "Static" then
						self.wti.statsSession['trees'][1] = self.wti.statsSession['trees'][1] + 1
						self.wti.statsSession['trees'][2] = self.wti.statsSession['trees'][2] + sizeX
						self.wti.statsSession['trees'][3] = self.wti.statsSession['trees'][3] + mass
						self.wti.statsSession['trees'][4] = self.wti.statsSession['trees'][4] + vol
						self.wti.statsTotal['trees'][1] = self.wti.statsTotal['trees'][1] + 1
						self.wti.statsTotal['trees'][2] = self.wti.statsTotal['trees'][2] + sizeX
						self.wti.statsTotal['trees'][3] = self.wti.statsTotal['trees'][3] + mass
						self.wti.statsTotal['trees'][4] = self.wti.statsTotal['trees'][4] + vol
					elseif self.rigidBodyTemp == "Dynamic" then
						self.wti.statsSession['cuts'][1] = self.wti.statsSession['cuts'][1] + 1
						self.wti.statsSession['cuts'][2] = self.wti.statsSession['cuts'][2] + (self.hasAttachedSplitShape == true and (self.sizeXTemp - sizeX) or sizeX)
						self.wti.statsSession['cuts'][3] = self.wti.statsSession['cuts'][3] + (self.hasAttachedSplitShape == true and (self.massTemp - mass) or mass)
						self.wti.statsSession['cuts'][4] = self.wti.statsSession['cuts'][4] + (self.hasAttachedSplitShape == true and (self.volTemp - vol) or vol)
						self.wti.statsTotal['cuts'][1] = self.wti.statsTotal['cuts'][1] + 1
						self.wti.statsTotal['cuts'][2] = self.wti.statsTotal['cuts'][2] + (self.hasAttachedSplitShape == true and (self.sizeXTemp - sizeX) or sizeX)
						self.wti.statsTotal['cuts'][3] = self.wti.statsTotal['cuts'][3] + (self.hasAttachedSplitShape == true and (self.massTemp - mass) or mass)
						self.wti.statsTotal['cuts'][4] = self.wti.statsTotal['cuts'][4] + (self.hasAttachedSplitShape == true and (self.volTemp - vol) or vol)
					end
				end
				if self.cutTimer > 1 then
					self.sizeXTemp = sizeX
					self.massTemp = mass
					self.volTemp = vol
					self.rigidBodyTemp = getRigidBodyType(shape)
					self.isCutting = true
				else
					if self.isCutting == true then
						self.wti.writeStats = true
						self.isCutting = false
					end
				end
			else
				self.wti.treeFound = false
			end;
			if self.wti.showStats == true and self.wti.treeFound == true then
				self.wti.hudInfoPosY = 0.203
				if self.wti.woodChipsOn == true then
					self.wti.hudInfoSizeY = 0.22
				else
					self.wti.hudInfoSizeY = 0.21
				end
			elseif self.wti.showStats == true then
				self.wti.hudInfoPosY = 0.203
				if self.wti.woodChipsOn == true then
					self.wti.hudInfoSizeY = 0.154
				else
					self.wti.hudInfoSizeY = 0.144
				end
			elseif self.wti.treeFound == true then
				self.wti.hudInfoPosY = 0.203
				self.wti.hudInfoSizeY = 0.063
			end
			if self.wti.showStats == true then
				self.wti.treeInfo = self.wti.treeInfo .. "Statistics:\n";
				self.wti.treeInfo = self.wti.treeInfo .. "----------------------------------------------------------\n";
				self.wti.treeInfo = self.wti.treeInfo .. "Trees down: \n";
				self.wti.treeInfo = self.wti.treeInfo .. "									Weight: \n";
				self.wti.treeInfo = self.wti.treeInfo .. "								Volume: \n";
				self.wti.treeInfo = self.wti.treeInfo .. "Logs cutted: \n";
				self.wti.treeInfo = self.wti.treeInfo .. "									Weight: \n";
				self.wti.treeInfo = self.wti.treeInfo .. "								Volume: \n";
				if self.wti.woodChipsOn == true then
					self.wti.treeInfo = self.wti.treeInfo .. "WoodChips: \n";
				end
				self.wti.treeInfo2 = self.wti.treeInfo2 .. "This Session\n\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. self.wti.statsSession['trees'][1] .. " (" .. string.format("%.0f", self.wti.statsSession['trees'][2]) .. "m)\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. string.format("%.1f", self.wti.statsSession['trees'][3]) .. " t\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. string.format("%.1f", self.wti.statsSession['trees'][4]) .. " m3\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. self.wti.statsSession['cuts'][1] .. " (" .. string.format("%.0f", self.wti.statsSession['cuts'][2]) .. "m)\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. string.format("%.1f", self.wti.statsSession['cuts'][3]) .. " t\n";
				self.wti.treeInfo2 = self.wti.treeInfo2 .. string.format("%.1f", self.wti.statsSession['cuts'][4]) .. " m3\n";
				if self.wti.woodChipsOn == true then
					self.wti.treeInfo2 = self.wti.treeInfo2 .. string.format("%.1f", self.wti.statsSession['cuts'][5]) .. " l\n";
				end
				self.wti.treeInfo4 = self.wti.treeInfo4 .. "Total\n\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. self.wti.statsTotal['trees'][1] .. " (" .. string.format("%.0f", self.wti.statsTotal['trees'][2]) .. "m)\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. string.format("%.1f", self.wti.statsTotal['trees'][3]) .. " t\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. string.format("%.1f", self.wti.statsTotal['trees'][4]) .. " m3\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. self.wti.statsTotal['cuts'][1] .. " (" .. string.format("%.0f", self.wti.statsTotal['cuts'][2]) .. "m)\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. string.format("%.1f", self.wti.statsTotal['cuts'][3]) .. " t\n";
				self.wti.treeInfo4 = self.wti.treeInfo4 .. string.format("%.1f", self.wti.statsTotal['cuts'][4]) .. " m3\n";
				if self.wti.woodChipsOn == true then
					self.wti.treeInfo4 = self.wti.treeInfo4 .. string.format("%.1f", self.wti.statsTotal['cuts'][5]) .. " l\n";
				end
			end
			shape = 0;
			
			if InputBinding.hasEvent(InputBinding.WTI_RESET_SESSION) then
				for k,v in pairs(self.wti.statsSession) do
					self.wti.statsSession[k] = {0,0,0,0,0}
				end
			end;
			if InputBinding.hasEvent(InputBinding.WTI_RESET_TOTAL) then
				for k,v in pairs(self.wti.statsTotal) do
					self.wti.statsTotal[k] = {0,0,0,0,0}
				end
			end;
			if InputBinding.hasEvent(InputBinding.WTI_SWITCH_STATS) then
				self.wti.showStats = not self.wti.showStats
				self.wti.treeInfo = "";
				self.wti.treeInfo2 = "";
				self.wti.treeInfo3 = "";
				self.wti.treeInfo4 = "";
			end;
			if self.wti.woodChipsOn == true then
				if g_currentMission.sawdustBase.woodHarvesterCounter > 0 then
					self.wti.isChipping = true
				else
					if self.wti.isChipping == true then
						self.wti.isChipping = false
						local woodChipsAmount = g_currentMission.sawdustBase:calcDelta(0)
						self.wti.statsSession['cuts'][5] = self.wti.statsSession['cuts'][5] + (woodChipsAmount / 1)
						self.wti.statsTotal['cuts'][5] = self.wti.statsTotal['cuts'][5] + (woodChipsAmount / 1)
					end
				end
			end
		end;
	end;
end;

function WoodHarvesterTreeInfo:updateTick(dt)
end;

function WoodHarvesterTreeInfo:draw()
	if self.isClient then
		if self:getIsActiveForInput(true) then
		if self:getIsTurnedOn() then
			if self.wti.treeFound == true or self.wti.showStats == true then
				self.wti.hud = Overlay:new("WoodHarvesterTreeInfo", Utils.getFilename("WoodHarvesterTreeInfo_hud.dds", WoodHarvesterTreeInfo_directory), self.wti.hudInfoPosX, self.wti.hudInfoPosY, self.wti.hudInfoSizeX, self.wti.hudInfoSizeY);
				self.wti.hud:render();
				setTextBold(false);
				setTextAlignment(RenderText.ALIGN_LEFT);
				setTextColor(0.5, 0.5, 0.5, 1);
				renderText(self.wti.hudInfoPosX + 0.005, self.wti.hudInfoPosY + self.wti.hudInfoSizeY - 0.018, 0.015, self.wti.treeInfo);
				setTextAlignment(RenderText.ALIGN_RIGHT);
				setTextColor(1,1,1,1);
				setTextBold(false);
				renderText(self.wti.hudInfoPosX + 0.105, self.wti.hudInfoPosY + self.wti.hudInfoSizeY - 0.018, 0.015, self.wti.treeInfo2);
				setTextBold(false);
				setTextAlignment(RenderText.ALIGN_LEFT);
				renderText(self.wti.hudInfoPosX + 0.11, self.wti.hudInfoPosY + self.wti.hudInfoSizeY - 0.018, 0.015, self.wti.treeInfo3);
				setTextBold(false);
				setTextAlignment(RenderText.ALIGN_RIGHT);
				renderText(self.wti.hudInfoPosX + 0.165, self.wti.hudInfoPosY + self.wti.hudInfoSizeY - 0.018, 0.015, self.wti.treeInfo4);
				setTextBold(false);
				setTextAlignment(RenderText.ALIGN_LEFT);
			end;
			g_currentMission:addHelpButtonText(g_i18n:getText("input_WTI_RESET_SESSION") , InputBinding.WTI_RESET_SESSION);
			g_currentMission:addHelpButtonText(g_i18n:getText("input_WTI_RESET_TOTAL") , InputBinding.WTI_RESET_TOTAL);
			if self.wti.showStats == true then
				g_currentMission:addHelpButtonText(g_i18n:getText("WTI_STATS_OFF") , InputBinding.WTI_SWITCH_STATS);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("WTI_STATS_ON") , InputBinding.WTI_SWITCH_STATS);
			end
		end;
		self.wti.treeInfo = "";
		self.wti.treeInfo2 = "";
		self.wti.treeInfo3 = "";
		self.wti.treeInfo4 = "";
		end;
	end;
end;

function WoodHarvesterTreeInfo:onLeave()
end;